require "wowhead_helper"
require "net/http"

#
# Please note for any given 'list' max returned items is 200
#
class WoWHeadMaps
  include JsonHelper
  include WoWHeadHelper
  def initialize()
    @@herbs = Hash.new
    @@minerals = Hash.new
    @@containers = Hash.new
    @@pickable = Hash.new
    @@fishes = Hash.new
    @@npcs = Hash.new
    @@zones = Hash.new
    @@raids = Hash.new
    @@instances = Hash.new
    get_zone_list
  end
  def load_all_resource_maps
    get_zone_list
    get_pickable_list
    get_container_list
    get_herb_list
    get_mineral_list
  end

  def get_pickable_list
    if @@pickable.length == 0
      picks = search_objects("-5")
      picks.each do |h|
        if not h[:skill].nil? and h[:skill] > 0 and (not h[:location].nil?)
          if @@pickable[h[:name]]
            @@pickable[h[:name]] << h[:id]
          else  
            @@pickable[h[:name]] = [h[:id]]
          end                    
        end
      end      
    end
    return @@pickable
  end
  def get_container_list
    if @@containers.length == 0
      chests = search_objects("3")
      chests.each do |h|
        next if h[:id].to_i == 210582
        unless h[:location].nil?
          if @@containers[h[:name]]
            @@containers[h[:name]] << h[:id]
          else  
            @@containers[h[:name]] = [h[:id]]
          end          
        end
      end      
    end
    return @@containers
  end
  # return the full list of herbs you can pick
  def get_herb_list
    if @@herbs.length == 0
      herbs = search_objects("-3")
      herbs.each do |h|
        if not h[:skill].nil? and h[:skill] > 0 and (not h[:location].nil?)
          if @@herbs[h[:name]]
            @@herbs[h[:name]] << h[:id]
          else  
            @@herbs[h[:name]] = [h[:id]]
          end
        end
      end      
    end
    return @@herbs
  end
  # return a full list of nodes you can mine
  def get_mineral_list
    # Mineral http://www.wowdb.com/search.aspx?browse=5.-1
    if @@minerals.length == 0
      minerals = search_objects("-4")
      puts minerals.inspect
      minerals.each do |m|
        if not m[:skill].nil? and m[:skill] > 0 and (not m[:location].nil?)
          if @@minerals[m[:name]]
            @@minerals[m[:name]] << m[:id]
          else  
            @@minerals[m[:name]] = [m[:id]]
          end
        end
      end
    end
    return @@minerals
  end
  
  def get_maps(object_name)
    if @@herbs.has_key?(object_name)
      result = {}
      @@herbs[object_name].each do |mid|
        p = get_map_locations(mid)
        puts p
        result.merge!(p)
      end
      return result      
    end
    if @@minerals.has_key?(object_name)
      result = {}
      @@minerals[object_name].each do |mid|
        p = get_map_locations(mid)
        puts p
        result.merge!(p)
      end
      return result
    end
    if @@containers.has_key?(object_name)
      result = {}
      @@containers[object_name].each do |mid|
        p = get_map_locations(mid)
        puts p
        result.merge!(p)
      end
      return result
    end
    if @@pickable.has_key?(object_name)
        result = {}
        @@pickable[object_name].each do |mid|
          p = get_map_locations(mid)
          puts p
          result.merge!(p)
        end
        return result
    end
    raise "Maps not found for #{object_name} either you mis-spelled the name or you have fetched the list yet"
  end
  def get_zone_list()
    response = Net::HTTP.get(URI.parse("http://www.wowhead.com/zones"))
    unless response
      raise "Failed to load zone list!"
    end
    lst = response.scan(/Listview.(.*).;/).first
    if lst.nil?
      raise "No search results found"
    end
    data = lst.first
    # need to manip the data  little
    data.gsub!('"category"','category')
    data.gsub!('"id"','id')
    data.gsub!('"instance"','instance')
    data.gsub!('"name"','name')
    data.gsub!('"minlevel"','minlevel')
    data.gsub!('"maxlevel"','maxlevel')
    data.gsub!('"nplayers"','nplayers')
    data.gsub!('"reqlevel"','reqlevel')
    data.gsub!('"territory"','territory')
    data.gsub!('"expansion"','expansion')
    data.gsub!('"playerlimit"','playerlimit')
    data.gsub!('"worldpvp"','worldpvp')
    data.gsub!('"lfgReqLevel"','lfgReqLevel')
    data.gsub!('"heroicLevel"','heroicLevel')
    zones = from_json(data)[:data]
    zones.each do |zone|
        @@zones[zone[:id]]=zone
        if zone[:category] == 2
          @@instances[zone[:id]] = zone
        end
        if zone[:category] == 3
          @@raids[zone[:id]] = zone
        end
    end
  end
  def get_raid_maps
    return @@raids
  end
  def get_dungeon_maps
    return @@instances
  end
  # returns a Hash keyed by Zone name, containing an array of coord arrays
  def get_map_locations(object_id)
    #puts "looking for #{object_id}"
    response = Net::HTTP.get(URI.parse("http://www.wowhead.com/object=#{object_id}"))
    unless response
      raise "Data not available for object #{object_id}"
    end
    #puts response
    # Updates to Wowhead new Format
    blob = response.scan(/g_mapperData\s+?=\s+?(.*?);/m)
    #puts blob
    links = blob.first.to_s
    links[0..1] = ""
    links.rstrip!
    links.gsub!("\\n","")
    links.gsub!("\"","")
    links.chop!
    puts links
    #links = links.scan(/(\d+)\:\s+?\{(.+?\s+?\}+?)?\s+?\}/m)
    links = links.scan(/(\d+)\:\s+?\{\s(.+?)\}\s\}/m)
    puts "Links #{links.inspect}"
    data = Array.new
    links.each do |lnk|
        zid = lnk[0]
        zdata = lnk[1]
        #puts "ZDATA #{zdata} for zone : #{zid}"
        if not zdata.nil? and zdata.include?("count")
          begin
            mdata = from_json_quoted("{#{zdata}}}")
            mdata.each_pair do |k,v|
              tmp = {}
              tmp[:coords] = v["coords"]
              tmp[:zone] = zid.to_i
              data << tmp
            end
          rescue Exception=>e
            puts "Error parsing #{e}"
          end
        end        
    end
    if @@zones.length == 0
        get_zone_list
    end
    # sort data into a map
    results = Hash.new
    data.each do |entry|
        zone = @@zones[entry[:zone]][:name]
        coords = entry[:coords]
        coords.each do |coordset|
          coordset.delete_if do |record|
            record.kind_of?(String)
          end
        end
        results[zone] = coords
      #end
    end
    return results
  end
end
